package com.dzeio.openhealth.core

import android.app.Dialog
import android.os.Bundle
import android.view.LayoutInflater
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment
import androidx.viewbinding.ViewBinding
import com.google.android.material.dialog.MaterialAlertDialogBuilder

/**
 * Base around the DialogFragment class to simplify usage
 */
abstract class BaseSimpleDialog<VB : ViewBinding> : DialogFragment() {

    /**
     * Function to inflate the Fragment Bindings
     */
    abstract val bindingInflater: (LayoutInflater) -> VB

    private var _binding: VB? = null
    val binding get() = _binding!!

    /**
     * Setup everything!
     */
    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        return activity?.let { act ->
            val builder = MaterialAlertDialogBuilder(requireContext())

            _binding = bindingInflater(act.layoutInflater)

            builder.setView(binding.root)

            onBuilderInit(builder)

            val dialog = builder.create()

            onDialogInit(dialog)

            onCreated()

            dialog
        } ?: throw IllegalStateException("Activity cannot be null")
    }

    /**
     * Function to add more customization to the AlertDialogBuilder
     */
    open fun onBuilderInit(builder: MaterialAlertDialogBuilder) {}

    /**
     * Function that allow to modificate some elements of the final dialog
     */
    open fun onDialogInit(dialog: AlertDialog) {}

    /**
     * Function run when the dialog is created
     */
    open fun onCreated() {}

    /**
     * Destroy binding
     */
    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}
