package com.dzeio.openhealth.data.step

import androidx.room.ColumnInfo
import androidx.room.Entity
import androidx.room.PrimaryKey
import java.text.DateFormat
import java.util.Calendar
import java.util.Date
import java.util.Locale
import java.util.TimeZone

@Entity()
data class Step(
    @PrimaryKey(autoGenerate = true) var id: Long = 0,

    /**
     * the raw number of step
     */
    var value: Int = 0,
    /**
     * Timestamp down to an hour
     *
     * ex: 2022-09-22 10:00:00
     */
    @ColumnInfo(index = true)
    var timestamp: Long = 0,

    /**
     * the source for the Entry
     *
     * note: Unused currently but kept for future usage
     */
    var source: String = "OpenHealth"
) {

    init {
        if (timestamp == 0L) {
            val cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"))
            cal.set(Calendar.MINUTE, 0)
            cal.set(Calendar.SECOND, 0)
            cal.set(Calendar.MILLISECOND, 0)

            timestamp = cal.timeInMillis
        }
    }

    fun formatTimestamp(removeTime: Boolean = false): String {
        val formatter = if (removeTime) {
            DateFormat.getDateInstance(
                DateFormat.SHORT,
                Locale.getDefault()
            )
        } else {
            DateFormat.getDateTimeInstance(
                DateFormat.SHORT,
                DateFormat.SHORT,
                Locale.getDefault()
            )
        }
        return formatter.format(Date(this.timestamp))
    }

    fun isToday(): Boolean {
        val it = getDay()

        val cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"))

        cal.set(Calendar.HOUR, 0)
        cal.set(Calendar.MINUTE, 0)
        cal.set(Calendar.SECOND, 0)
        cal.set(Calendar.MILLISECOND, 0)
        return it == cal.timeInMillis
    }

    fun getDay(): Long {
        val it = Calendar.getInstance(TimeZone.getTimeZone("UTC")).apply {
            timeInMillis = timestamp
            set(Calendar.HOUR, 0)

            set(Calendar.AM_PM, Calendar.AM)
        }
        return it.timeInMillis
    }

    fun isCurrent(): Boolean {
        val cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"))
        cal.set(Calendar.MINUTE, 0)
        cal.set(Calendar.SECOND, 0)
        cal.set(Calendar.MILLISECOND, 0)
        return timestamp == cal.timeInMillis
    }
}
