package com.dzeio.openhealth.devices

import java.util.UUID

object BluetoothLeGattUuid {
    private const val STANDARD_SUFFIX = "-0000-1000-8000-00805f9b34fb"
    fun fromShortCode(code: Long): UUID {
        return UUID.fromString(String.format("%08x%s", code, STANDARD_SUFFIX))
    }

    // https://www.bluetooth.com/specifications/gatt/services
    val SERVICE_GENERIC_ACCESS = fromShortCode(0x1800) // 1800
    val SERVICE_GENERIC_ATTRIBUTE = fromShortCode(0x1801) // 1801

    val SERVICE_CURRENT_TIME = fromShortCode(0x1805)
    val SERVICE_DEVICE_INFORMATION = fromShortCode(0x180A)
    val SERVICE_BATTERY_LEVEL = fromShortCode(0x180F)
    val SERVICE_BODY_COMPOSITION = fromShortCode(0x181B) // 181B
    val SERVICE_USER_DATA = fromShortCode(0x181C)
    val SERVICE_WEIGHT_SCALE = fromShortCode(0x181D)

    // https://www.bluetooth.com/specifications/gatt/characteristics
    val CHARACTERISTIC_DEVICE_NAME = fromShortCode(0x2A00)
    val CHARACTERISTIC_APPEARANCE = fromShortCode(0x2A01)
    val CHARACTERISTIC_PERIPHERAL_PRIVACY_FLAG = fromShortCode(0x2A02)
    val CHARACTERISTIC_RECONNECTION_ADDRESS = fromShortCode(0x2A03)
    val CHARACTERISTIC_PERIPHERAL_PREFERRED_CONNECTION_PARAMETERS = fromShortCode(0x2A04)
    val CHARACTERISTIC_SERVICE_CHANGED = fromShortCode(0x2A05)

    val CHARACTERISTIC_BATTERY_LEVEL = fromShortCode(0x2A19)

    val CHARACTERISTIC_SYSTEM_ID = fromShortCode(0x2A23)
    val CHARACTERISTIC_MODEL_NUMBER_STRING = fromShortCode(0x2A24)
    val CHARACTERISTIC_SERIAL_NUMBER_STRING = fromShortCode(0x2A25)
    val CHARACTERISTIC_FIRMWARE_REVISION_STRING = fromShortCode(0x2A26)
    val CHARACTERISTIC_HARDWARE_REVISION_STRING = fromShortCode(0x2A27)
    val CHARACTERISTIC_SOFTWARE_REVISION_STRING = fromShortCode(0x2A28)
    val CHARACTERISTIC_MANUFACTURER_NAME_STRING = fromShortCode(0x2A29)
    val CHARACTERISTIC_IEEE_11073_20601_REGULATORY_CERTIFICATION_DATA_LIST = fromShortCode(0x2A2A)
    val CHARACTERISTIC_CURRENT_TIME = fromShortCode(0x2A2B)

    val CHARACTERISTIC_PNP_ID = fromShortCode(0x2A50)

    val CHARACTERISTIC_USER_AGE = fromShortCode(0x2A80)
    val CHARACTERISTIC_USER_DATE_OF_BIRTH = fromShortCode(0x2A85)
    val CHARACTERISTIC_USER_GENDER = fromShortCode(0x2A8C)
    val CHARACTERISTIC_USER_HEIGHT = fromShortCode(0x2A8E)

    val CHARACTERISTIC_CHANGE_INCREMENT = fromShortCode(0x2A99)
    val CHARACTERISTIC_BODY_COMPOSITION_MEASUREMENT = fromShortCode(0x2A9C)
    val CHARACTERISTIC_WEIGHT_MEASUREMENT = fromShortCode(0x2A9D)
    val CHARACTERISTIC_USER_CONTROL_POINT = fromShortCode(0x2A9F)

    // https://www.bluetooth.com/specifications/gatt/descriptors
    val DESCRIPTOR_CLIENT_CHARACTERISTIC_CONFIGURATION = fromShortCode(0x2902)
    val DESCRIPTOR_CHARACTERISTIC_USER_DESCRIPTION = fromShortCode(0x2901)
}
