package com.dzeio.openhealth.ui.browse

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.dzeio.openhealth.Settings
import com.dzeio.openhealth.core.BaseViewModel
import com.dzeio.openhealth.data.step.StepRepository
import com.dzeio.openhealth.data.weight.WeightRepository
import com.dzeio.openhealth.units.Mass
import com.dzeio.openhealth.utils.Configuration
import dagger.hilt.android.lifecycle.HiltViewModel
import javax.inject.Inject
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

@HiltViewModel
class BrowseViewModel @Inject internal constructor(
    stepRepository: StepRepository,
    weightRepository: WeightRepository,
    config: Configuration
) : BaseViewModel() {

    private val _massUnit = MutableLiveData(Mass.KILOGRAM)
    val massUnit: LiveData<Mass> = _massUnit

    private val _steps = MutableLiveData(0)
    val steps: LiveData<Int> = _steps

    val stepsGoal = config.getInt(Settings.STEPS_GOAL).toLiveData()

    private val _weight = MutableLiveData(0f)
    val weight: LiveData<Float> = _weight

    init {
        viewModelScope.launch {
            _steps.postValue(stepRepository.todaySteps())
        }

        viewModelScope.launch {
            weightRepository.lastWeight().collectLatest {
                if (it == null) {
                    return@collectLatest
                }
                _weight.postValue(it.weight)
            }
        }

        config.getInt(Settings.MASS_UNIT).apply {
            addObserver {
                if (it == null) return@addObserver
                _massUnit.postValue(Mass.find(it))
            }
            if (value != null) {
                _massUnit.postValue(Mass.find(value!!))
            }
        }
    }
}
