package com.dzeio.openhealth.ui.steps

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.dzeio.openhealth.core.BaseViewModel
import com.dzeio.openhealth.data.food.Food
import com.dzeio.openhealth.data.food.FoodRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import java.util.Calendar
import javax.inject.Inject
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

@HiltViewModel
class FoodHomeViewModel @Inject internal constructor(
    private val foodRepository: FoodRepository
) : BaseViewModel() {
    val items: MutableLiveData<List<Food>> = MutableLiveData()
    private val list: MutableLiveData<List<Food>> = MutableLiveData(arrayListOf())
    val date: MutableLiveData<Long> = MutableLiveData(Calendar.getInstance().timeInMillis)

    fun init() {
        val now = Calendar.getInstance()
        now.set(Calendar.HOUR, 0)
        now.set(Calendar.MINUTE, 0)
        now.set(Calendar.SECOND, 0)
        date.postValue(now.timeInMillis)
        viewModelScope.launch {
            foodRepository.getAll().collectLatest {
                list.postValue(it)
                updateList(it)
            }
        }
    }

    fun next() {
        val now = Calendar.getInstance()
        now.timeInMillis = date.value!!
        now.add(Calendar.DAY_OF_YEAR, 1)

        date.value = now.timeInMillis

        updateList()
    }

    fun previous() {
        val now = Calendar.getInstance()
        now.timeInMillis = date.value!!
        now.add(Calendar.DAY_OF_YEAR, -1)

        date.value = now.timeInMillis

        updateList()
    }

    private fun updateList(foods: List<Food>? = null) {
        val day = Calendar.getInstance()
        day.timeInMillis = date.value!!
        val todayInMillis = day.timeInMillis
        day.add(Calendar.DAY_OF_YEAR, 1)
        val tomorrow = day.timeInMillis

        items.postValue(
            (foods ?: list.value!!).filter { food ->
                food.timestamp in todayInMillis until tomorrow
            }
        )
    }
}
