package com.dzeio.openhealth.ui.importexport

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentImportExportBinding
import dagger.hilt.android.AndroidEntryPoint
import java.util.zip.ZipInputStream
import kotlin.system.exitProcess

@AndroidEntryPoint
class ImportExportFragment : BaseFragment<ImportExportViewModel, FragmentImportExportBinding>(
    ImportExportViewModel::class.java
) {
    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentImportExportBinding =
        FragmentImportExportBinding::inflate

    /**
     * Response to the import button event
     */
    private val readResult = registerForActivityResult(
        ActivityResultContracts.OpenDocument()
    ) {
        // stop if no document was chosen
        if (it == null) {
            return@registerForActivityResult
        }

        // read the zipfile
        val stream = requireActivity().contentResolver.openInputStream(it)
        val zip = ZipInputStream(stream)
        viewModel.appDatabase.importDatabase(requireContext(), zip)

        // Restart the app
        val i = requireContext().packageManager.getLaunchIntentForPackage(requireContext().packageName)
        i!!.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
        requireContext().startActivity(i)
        Toast.makeText(requireContext(), R.string.import_complete, Toast.LENGTH_LONG).show()
        exitProcess(0)
    }

    /**
     * Response to the export button event
     */
    private val writeResult = registerForActivityResult(
        ActivityResultContracts.CreateDocument("application/zip")
    ) {

        // stop if no location is given
        if (it == null) {
            return@registerForActivityResult
        }

        // export the db to a zip
        val bkp = viewModel.appDatabase.exportDatabase(requireContext())
        viewModel.appDatabase.checkpoint()

        // write file to location
        requireActivity().contentResolver.openOutputStream(it)?.apply {
            write(bkp)
            close()
        }

        Toast.makeText(requireContext(), R.string.export_complete, Toast.LENGTH_LONG).show()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        binding.imports.setOnClickListener {
            // ask user the location of the backup file
            readResult.launch(arrayOf("application/zip"))
        }

        binding.export.setOnClickListener {
            // ask user the location to save the backup file
            writeResult.launch("export.openhealth")
        }
    }
}
