package com.dzeio.openhealth.ui.water

import android.view.LayoutInflater
import android.view.View
import androidx.core.widget.doOnTextChanged
import com.dzeio.openhealth.R
import com.dzeio.openhealth.Settings
import com.dzeio.openhealth.core.BaseDialog
import com.dzeio.openhealth.databinding.DialogWaterSizeSelectorBinding
import com.dzeio.openhealth.units.Volume
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class WaterSizeSelectorDialog :
    BaseDialog<WaterSizeSelectorViewModel, DialogWaterSizeSelectorBinding>(
        WaterSizeSelectorViewModel::class.java
    ) {

    override val bindingInflater: (LayoutInflater) -> DialogWaterSizeSelectorBinding
        get() = DialogWaterSizeSelectorBinding::inflate

    override fun onCreated() {
        super.onCreated()

        binding.cancel.setOnClickListener {
            dismiss()
        }

        binding.validate.setOnClickListener {
            dismiss()
        }

        viewModel.cupSize.observe(this) {
            setCustomSizeText()
        }

        viewModel.unit.observe(this) {
            if (it == null) {
                return@observe
            }
            setCustomSizeText()
            binding.size1000ml.text = getString(
                it.unit,
                it.fromMilliliter(1000)
            )
            binding.size100ml.text = getString(
                it.unit,
                it.fromMilliliter(100)
            )
            binding.size250ml.text = getString(
                it.unit,
                it.fromMilliliter(250)
            )
            binding.size500ml.text = getString(
                it.unit,
                it.fromMilliliter(500)
            )
        }

        binding.size100ml.setOnClickListener {
            setTextFieldStatus()
            viewModel.setCupSize(100)
        }
        binding.size250ml.setOnClickListener {
            setTextFieldStatus()
            viewModel.setCupSize(250)
        }
        binding.size500ml.setOnClickListener {
            setTextFieldStatus()
            viewModel.setCupSize(500)
        }
        binding.size1000ml.setOnClickListener {
            setTextFieldStatus()
            viewModel.setCupSize(1000)
        }

        binding.customSize.setOnClickListener {
            setTextFieldStatus(true)
        }

        binding.input.doOnTextChanged { text, _, _, _ ->
            val newSize = text.toString().toIntOrNull()
            if (newSize != null && binding.inputParent.visibility == View.VISIBLE) {
                val unit = viewModel.unit.value ?: Volume.MILLILITER
                viewModel.setCupSize(unit.toMilliliter(newSize))
            }
        }
    }

    private fun setTextFieldStatus(displayed: Boolean = false) {
        binding.inputParent.visibility = if (displayed) View.VISIBLE else View.GONE
        binding.customSize.visibility = if (displayed) View.GONE else View.VISIBLE
    }

    private fun setCustomSizeText() {
        val size = viewModel.cupSize.value ?: Settings.WATER_INTAKE_SIZE_DEFAULT
        val unit = viewModel.unit.value ?: Volume.MILLILITER

        val modified = unit.fromMilliliter(size)

        binding.customSize.text = String.format(
            getString(
                R.string.custom_amount,
                getString(unit.unit, modified)
            )
        )

        if (binding.inputParent.visibility == View.GONE) {
            binding.input.setText(modified.toString())
        }
    }
}
