package com.dzeio.openhealth.ui.weight

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.dzeio.openhealth.Settings
import com.dzeio.openhealth.core.BaseViewModel
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.data.weight.WeightRepository
import com.dzeio.openhealth.units.Mass
import com.dzeio.openhealth.utils.Configuration
import dagger.hilt.android.lifecycle.HiltViewModel
import javax.inject.Inject
import kotlin.random.Random
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

@HiltViewModel
class ListWeightViewModel @Inject internal constructor(
    private val weightRepository: WeightRepository,
    settings: Configuration
) : BaseViewModel() {

    private val _massUnit = MutableLiveData(Mass.KILOGRAM)
    val massUnit: LiveData<Mass> = _massUnit

    private val _goalWeight = settings.getFloat(Settings.WEIGHT_GOAL)
    val goalWeight = _goalWeight.toLiveData()

    private val _weights = MutableLiveData<List<Weight>?>(null)
    val weights: LiveData<List<Weight>?> = _weights

    init {
        viewModelScope.launch {
            weightRepository.getWeights().collectLatest {
                _weights.postValue(it)
            }
        }

        settings.getInt(Settings.MASS_UNIT).apply {
            addObserver {
                if (it == null) return@addObserver
                _massUnit.postValue(Mass.find(it))
            }
            if (value != null) {
                _massUnit.postValue(Mass.find(value!!))
            }
        }
    }

    fun generateRandomValues() {
        viewModelScope.launch {
            weightRepository.addWeight(
                Weight(
                    weight = Random.nextInt(0, 100).toFloat()
                )
            )
        }
    }

    fun delete(list: List<Weight>) {
        viewModelScope.launch {
            weightRepository.deleteWeight(*list.toTypedArray())
        }
    }
}
