package com.dzeio.openhealth.ui.weight

import android.annotation.SuppressLint
import android.app.ProgressDialog
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.widget.Toast
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.openhealth.R
import com.dzeio.openhealth.adapters.ItemAdapter
import com.dzeio.openhealth.core.BaseDialog
import com.dzeio.openhealth.databinding.DialogSearchBinding
import com.dzeio.openhealth.devices.BluetoothLeGattDevice
import com.dzeio.openhealth.devices.Device
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.launch

@AndroidEntryPoint
class ScanScalesDialog :
    BaseDialog<ScanScalesViewModel, DialogSearchBinding>(ScanScalesViewModel::class.java) {

    override val bindingInflater: (LayoutInflater) -> DialogSearchBinding =
        DialogSearchBinding::inflate

    @SuppressLint("MissingPermission")
    override fun onBuilderInit(builder: MaterialAlertDialogBuilder) {
        super.onBuilderInit(builder)

        builder.apply {
            setTitle(R.string.searching_scales)
            setIcon(R.drawable.ic_outline_monitor_weight_24)
            setNegativeButton(R.string.cancel) { dialog, _ ->
                viewModel.stopScan()
                dialog.cancel()
            }
        }
    }

    @SuppressLint("MissingPermission")
    override fun onCreated() {
        super.onCreated()
        binding.search.visibility = View.GONE
        binding.loading.visibility = View.VISIBLE

        val adapter = ItemAdapter<BluetoothLeGattDevice>().apply {
            onItemClick = { deviceAdapter ->
                dialog!!.dismiss()
                val device = deviceAdapter.value
                viewModel.stopScan()
                val progress = ProgressDialog(requireContext()).apply {
                    setTitle("Connecting & Fetching...")
                    setCancelable(false)
                    setProgressStyle(ProgressDialog.STYLE_HORIZONTAL)
                    show()
                }
                device.connect().addObserver { status ->
                    if (status === Device.ConnectionStatus.CONNECTED) {
                        device.fetchWeights().addObserver {
                            progress.progress = it.progress
                            progress.max = it.progressMax + 1

                            if (it.progress == it.progressMax) {
                                device.close()
                                Log.d("YAY", "${it.data}")
                                lifecycleScope.launch {
                                    viewModel.addWeights(it.data)
                                    progress.dismiss()
                                    Toast.makeText(
                                        requireContext(),
                                        "Data synchonised with the remote device",
                                        Toast.LENGTH_SHORT
                                    ).show()
                                }
                            }
                        }
                    } else if (status === Device.ConnectionStatus.ERROR) {
                        progress.dismiss()
                        Toast.makeText(
                            requireContext(),
                            "An error occured while connecting...",
                            Toast.LENGTH_LONG
                        ).show()
                    }
                }
            }
        }
        binding.list.apply {
            layoutManager = LinearLayoutManager(requireContext())
            this.adapter = adapter
        }

        viewModel.devices.observe(this) {
            if (it == null) {
                adapter.clear()
                return@observe
            }
            adapter.set(
                it.map {
                    ItemAdapter.Item(
                        it,
                        it.name,
                        "${it.item!!.name} (${it.item!!.address})",
                        icon = R.drawable.vector_add
                    )
                }
            )
        }
    }
}
