package com.dzeio.openhealth.ui.weight

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.dzeio.openhealth.Settings
import com.dzeio.openhealth.core.BaseViewModel
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.data.weight.WeightRepository
import com.dzeio.openhealth.units.ActivityLevel
import com.dzeio.openhealth.units.Mass
import com.dzeio.openhealth.utils.Configuration
import dagger.hilt.android.lifecycle.HiltViewModel
import javax.inject.Inject
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

@HiltViewModel
class WeightDialogViewModel @Inject internal constructor(
    private val weightRepository: WeightRepository,
    settings: Configuration
) : BaseViewModel() {

    private val _goalWeight = settings.getFloat(Settings.WEIGHT_GOAL)
    private val age = settings.getInt(Settings.USER_AGE)
    private val activityLevel = settings.getInt(Settings.USER_ACTIVITY_LEVEL)
    private val biologicalSex = settings.getInt(Settings.USER_BIOLOGICAL_SEX)
    private val height = settings.getInt(Settings.USER_HEIGHT)

    val goalWeight = _goalWeight.toLiveData()

    private val _weight = MutableLiveData<Weight?>(null)
    val weight: LiveData<Weight?> = _weight

    val format =
        Mass.find(settings.getInt(Settings.MASS_UNIT).value ?: Mass.KILOGRAM.ordinal)

    fun initWithWeight(id: Long?) {
        viewModelScope.launch {
            if (id != null) {
                weightRepository.getWeight(id).collectLatest {
                    _weight.postValue(it)
                }
            } else {
                weightRepository.lastWeight().collectLatest {
                    _weight.postValue(it)
                }
            }
        }
    }

    suspend fun addWeight(weight: Float) {
        weightRepository.addWeight(
            Weight(weight = weight / format.modifier).apply {
                if (height.value != null) {
                    setItemsWithEstimation(
                        height.value!!,
                        age.value,
                        biologicalSex.value,
                        if (activityLevel.value != null) ActivityLevel.values()[activityLevel.value!!] else null
                    )
                }
            }
        )
    }

    fun setWeightGoal(value: Float?) {
        if (value != null) {
            _goalWeight.value = value / format.modifier
        } else {
            _goalWeight.value = null
        }
    }
}
