package com.dzeio.openhealth.units

import com.dzeio.openhealth.R

/**
 * the Volume unit
 */
enum class Volume(
    /**
     * Value based on the Milliliter
     */
    val modifier: Float,
    val singular: Int,
    val plural: Int,
    val unit: Int
) {
    MILLILITER(
        1f,
        R.string.unit_volume_milliliter_name_singular,
        R.string.unit_volume_milliliter_name_plural,
        R.string.unit_volume_milliliter_unit
    ),
    IMPERIAL_OUNCE(
        0.03519503f,
        R.string.unit_volume_imperial_ounce_name_singular,
        R.string.unit_volume_imperial_ounce_name_plural,
        R.string.unit_volume_ounce_unit
    ),
    US_OUNCE(
        0.03381413f,
        R.string.unit_volume_us_ounce_name_singular,
        R.string.unit_volume_us_ounce_name_plural,
        R.string.unit_volume_ounce_unit
    );

    companion object {
        fun find(index: Int): Volume {
            return Volume.values().getOrNull(index) ?: MILLILITER
        }
    }

    /**
     * transform the value from the specified unit to kilograms
     */
    fun toMilliliter(value: Float): Float = value / modifier
    fun toMilliliter(value: Int): Int = toMilliliter(value.toFloat()).toInt()

    /**
     * transform the value from the specified unit to kilograms
     */
    fun fromMilliliter(value: Float): Float = value * modifier
    fun fromMilliliter(value: Int): Int = fromMilliliter(value.toFloat()).toInt()
}
